/*-------------------------------------------------------------------------
 *
 * logtofile.h
 *      Main entry point for logtofile
 *
 * Copyright (c) 2020-2026, Francisco Miguel Biete Banon
 *
 * This code is released under the PostgreSQL licence, as given at
 *  http://www.postgresql.org/about/licence/
 *-------------------------------------------------------------------------
 */
#ifndef _LOGTOFILE_H_
#define _LOGTOFILE_H_

void _PG_init(void);
void _PG_fini(void);

#endif
