/*-------------------------------------------------------------------------
 *
 * logtofile_string_format.h
 *      Functions to format data as strings
 *
 * Copyright (c) 2020-2026, Francisco Miguel Biete Banon
 *
 * This code is released under the PostgreSQL licence, as given at
 *  http://www.postgresql.org/about/licence/
 *-------------------------------------------------------------------------
 */
#ifndef _LOGTOFILE_STRING_FORMAT_H_
#define _LOGTOFILE_STRING_FORMAT_H_

#include <postgres.h>
#include <datatype/timestamp.h>
#include <portability/instr_time.h>

#define FORMATTED_TS_LEN 64
#define FORMATTED_NUMLINE_LEN 32

extern void PgAuditLogToFile_format_instr_time_nanos(instr_time t, char *buf, size_t len);


#endif
